-- Copyright 2016, 2017 Cumulus Networks, Inc.  All Rights reserved.
--
-- This software is subject to the Cumulus Networks End User License Agreement
-- available at the following locations:
--
-- Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
-- Cumulus Linux systems: /usr/share/cumulus/EULA.txt

local function _mysplit(inputstr, sep)
    local fields={}
    local pattern = string.format("([^%s]+)", sep)
    for str in string.gmatch(inputstr, pattern) do
        fields[#fields + 1] = str
    end
    return fields
end

local function _get_ipv6_subnet(ip, mask)
    local tmp = 128
    local tmptbl = _mysplit(ip, ":")
    local op = {}
    for idx = 1, #tmptbl do
        if tmp - 16 >= mask then
            op[#op + 1] = tonumber(tmptbl[idx], 16)
        elseif tmp > mask and tmp - 16 < mask then
          local netmask = 0
          for idx1 = 16, 16 - (tmp - mask), -1 do
            netmask = bit.bor(netmask, bit.blshift(1, idx1))
          end
          op[#op + 1] = bit.band(tonumber(tmptbl[idx], 16), netmask)
        else
          op[#op + 1] = 0
        end
        tmp = tmp - 16
    end
    return table.concat(op, ':', 1, #op)
end

local function _ipv4tonumber(input)
    local ip1, ip2, ip3, ip4 = string.match(
        input,
        "(%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?)"
    )
    return 2 ^ 24 * ip1 + 2 ^ 16 * ip2 + 2 ^ 8 * ip3 + ip4
end

local function _is_ipv4_match(prefix1, prefix2, mask)
    local prefix_number = _ipv4tonumber(prefix1)
    local input_number = _ipv4tonumber(prefix2)
    local netmask = 0
    for i = 32 - mask, 32 do
        netmask = bit.bor(netmask, bit.blshift(1, i))
    end
    return bit.band(prefix_number, netmask) ==
           bit.band(input_number, netmask)
end

local function expand_ipv6(ip)
    local first, last = string.match(ip, "(.*)::(.*)")
    if first == nil then
       first = ip
    end
    -- Initialize result
    local res = {}
    for idx=1, 8 do
       res[idx] = 0
    end

    local tmptbl
    local mul = 1
    tmptbl = _mysplit(first, ":")
    for idx = 1, #tmptbl do
       res[mul] = tmptbl[idx]
       mul = mul + 1
    end
    mul = 8
    if last ~= nil then
       tmptbl = _mysplit(last, ":")
       for idx = #tmptbl, 1, -1 do
       	   res[mul] = tmptbl[idx]
	   mul = mul - 1
       end
    end
    return table.concat(res, ':', 1, #res)
end

local function _is_ipv6_match(ip1, ip2, mask)
    return _get_ipv6_subnet(expand_ipv6(ip1), mask) ==
           _get_ipv6_subnet(expand_ipv6(ip2), mask)
end

local function is_match(prefix1, prefix2, mask, is_ipv6)
    if mask == 0 then
       return true
    end
    if is_ipv6 == 1 then
        return _is_ipv6_match(prefix1, prefix2, mask)
    else
        return _is_ipv4_match(prefix1, prefix2, mask)
    end
end

local function sub_prefix(input, is_ipv6)
  local rstr = nil
  if is_ipv6 == 1 then
    if input == '::/0' then
      return nil
    end
    local first, last = string.match(input, "(.*)::(.*)")
    if first ~= nil then
      if last ~= '*' then
        rstr = first..'::*'
      else
        local pre, post = string.match(first, "(.*):(.*)")
        if pre ~= nil then
          rstr = pre..'::*'
        end
      end
    end
    if rstr == nil then
      rstr = '::/0'
    end
  else
    if input == '0.0.0.0/0' then
      return nil
    end
    rstr = string.gsub(input, "(%.?%d%d?%d?%*?)$", "*")
    if rstr == '*' then
      rstr = '0.0.0.0/0'
    end
  end
  return rstr
end

local utils = {
    -- utility func
    expand_ipv6 = expand_ipv6,
    is_match = is_match,
    sub_prefix = sub_prefix,
}
